@@ -174,7 +174,7 @@ impl KeyPublicBytes for X25519KeyPair {
 }
 
 impl ToJwk for X25519KeyPair {
-    fn encode_jwk(&self, enc: &mut JwkEncoder<'_>) -> Result<(), Error> {
+    fn encode_jwk(&self, enc: &mut dyn JwkEncoder) -> Result<(), Error> {
         enc.add_str("crv", JWK_CURVE)?;
         enc.add_str("kty", JWK_KEY_TYPE)?;
         self.with_public_bytes(|buf| enc.add_as_base64("x", buf))?;
@@ -193,6 +193,12 @@ impl ToJwk for X25519KeyPair {
 
 impl FromJwk for X25519KeyPair {
     fn from_jwk_parts(jwk: JwkParts<'_>) -> Result<Self, Error> {
+        if jwk.kty != JWK_KEY_TYPE {
+            return Err(err_msg!(InvalidKeyData, "Unsupported key type"));
+        }
+        if jwk.crv != JWK_CURVE {
+            return Err(err_msg!(InvalidKeyData, "Unsupported key algorithm"));
+        }
         ArrayKey::<U32>::temp(|pk_arr| {
             if jwk.x.decode_base64(pk_arr)? != pk_arr.len() {
                 Err(err_msg!(InvalidKeyData))
@@ -258,7 +264,7 @@ mod tests {
             .to_jwk_public(None)
             .expect("Error converting public key to JWK");
         let jwk = JwkParts::from_str(&jwk).expect("Error parsing JWK output");
-        assert_eq!(jwk.kty, "OKP");
+        assert_eq!(jwk.kty, JWK_KEY_TYPE);
         assert_eq!(jwk.crv, JWK_CURVE);
         assert_eq!(jwk.x, "tGskN_ae61DP4DLY31_fjkbvnKqf-ze7kA6Cj2vyQxU");
         assert_eq!(jwk.d, None);
@@ -266,7 +272,7 @@ mod tests {
         assert_eq!(kp.to_public_bytes(), pk_load.to_public_bytes());
 
         let jwk = kp
-            .to_jwk_secret()
+            .to_jwk_secret(None)
             .expect("Error converting private key to JWK");
         let jwk = JwkParts::from_slice(&jwk).expect("Error parsing JWK output");
         assert_eq!(jwk.kty, JWK_KEY_TYPE);
